#!/usr/bin/perl -w
# A quick and dirty web browser demonstrating the direct instantiation 
# of a KHTML part. Needless to say: this is NOT a programming paragon :)
# -- gg.

use strict;
use KDE;

package PartHolder;
use Qt;
use Qt::isa "Qt::Object";
use Qt::attributes qw(
     m_part
     m_history
);

sub NEW { 
    shift->SUPER::NEW( @_ ); 
    m_part = shift; 
    m_history = [];
}

sub setLocBarText: SIGNAL( const QString & );

sub openURL: SLOT( const KURL & )
{
    m_part->openURL( $_[0] );
    emit setLocBarText( $_[0]->url );
    unshift @{ &m_history }, $_[0] 
            unless $_[0] == m_history->[0];
}

sub reload(): SLOT()
{
    m_part->openURL( m_part->url() );
}

sub goToURL: SLOT(const QString&)
{
    my $url = shift;
    $url = "http://" . $url unless $url =~ /^\w*:/;
    openURL( KDE::URL($url) );
}

sub back: SLOT()
{
    return unless @{ &m_history } > 1;
    shift @{ &m_history };
    openURL( m_history->[0] ) if @{ &m_history };
}

1;

use KDE;
use PartHolder;

use constant LOC_ED => 322;
use constant ERASE_B => 323;
use constant BACK_B => 324;

my $opt = [ [ "+[url]",      "An URL to open at startup.",          0            ],
            [ "z",           "A dummy binary option.",              0            ],
            [ "baz <file>",  "A long option with arg.",             0            ],
            [ "boz <file>",  "Same as above with default value",   "default.txt" ],
          ];            

my $about = KDE::AboutData("kludgeror", "Kludgeror", "0.1", "A basic web browser");
KDE::CmdLineArgs::init(\@ARGV, $about);
KDE::CmdLineArgs::addCmdLineOptions($opt);

my $args = KDE::CmdLineArgs::parsedArgs();
my $url = $args->count() ? $args->url(0) : KDE::URL("http://perlqt.sf.net"); 

print "Dummy z option activated.\n" if $args->isSet("z");
print "Dummy baz option has value: ".$args->getOption("baz")."\n" if $args->isSet("baz");

my $a = KDE::Application();

my $toplevel = KDE::MainWindow();
my $doc = KDE::HTMLPart( $toplevel, undef, $toplevel, undef, &KDE::HTMLPart::BrowserViewGUI );
my $ph = PartHolder( $doc );

Qt::Object::connect( $doc->browserExtension(), SIGNAL "openURLRequest( const KURL &, const KParts::URLArgs & )",
                      $ph, SLOT "openURL( const KURL& ) ");

$ph->openURL( $url );
$toplevel->setCentralWidget( $doc->widget() );
$toplevel->resize(700, 500);

{
  my ($d, $viewMenu,  $fileMenu, $locBar, $e);
  $d = $doc->domDocument();
    $viewMenu = $d->documentElement()->firstChild()->childNodes()->item( 2 )->toElement();
        $e = $d->createElement( "action" );
        $e->setAttribute( "name", "debugRenderTree" );
    $viewMenu->appendChild( $e );
        $e = $d->createElement( "action" );
        $e->setAttribute( "name", "debugDOMTree" );
    $viewMenu->appendChild( $e );
    $fileMenu = $d->documentElement()->firstChild()->firstChild()->toElement();
    $fileMenu->appendChild( $d->createElement( "separator" ) );   
        $e = $d->createElement( "action" );
        $e->setAttribute( "name", "exit" );
    $fileMenu->appendChild( $e );
    $locBar = $d->createElement( "toolbar" );
    $locBar->setAttribute( "name", "locationBar" );
        $e = $d->createElement( "action" );
        $e->setAttribute( "name", "reload" );
    $locBar->appendChild( $e );
  $d->documentElement()->appendChild( $locBar );
}

my $a1 = KDE::Action( "Reload", "reload", KDE::Shortcut(&Qt::Key_F5), $ph, SLOT "reload() ", $doc->actionCollection(), "reload" );
my $a2 = KDE::Action( "Exit", "exit", KDE::Shortcut(0), $a, SLOT "quit()",  $doc->actionCollection(), "exit" );

$toplevel->guiFactory()->addClient( $doc );

my $locBar    = $toplevel->toolBar("locationBar");
   $locBar->insertButton( "back", BACK_B, SIGNAL "clicked()", $ph, SLOT "back()", 1, "Go back" );
   $locBar->insertLined ($url->url(), LOC_ED, SIGNAL "returnPressed(const QString&)", $ph, SLOT "goToURL(const QString&)", 1, "Location");
   $locBar->insertButton( "locationbar_erase", ERASE_B, SIGNAL "clicked()", $locBar->getLined(LOC_ED), SLOT "clear()", 1, "Erase the location bar's content", 2 );
   $locBar->setItemAutoSized( LOC_ED, 1 );
   $locBar->getLined(LOC_ED)->createPopupMenu();
my $comp = $locBar->getLined(LOC_ED)->completionObject();
   $comp->setCompletionMode(&KDE::GlobalSettings::CompletionPopupAuto);

Qt::Object::connect($locBar->getLined(LOC_ED),SIGNAL "returnPressed(const QString&)",$comp,SLOT "addItem(const QString&)");
Qt::Object::connect( $ph, SIGNAL "setLocBarText(const QString &)", $locBar->getLined(LOC_ED), SLOT "setText(const QString &)");

$doc->setJScriptEnabled(1);
$doc->setJavaEnabled(1);
$doc->setPluginsEnabled(1);
$doc->setURLCursor(Qt::Cursor(&Qt::PointingHandCursor));

$a->setTopWidget($doc->widget());

Qt::Object::connect($doc, SIGNAL "setWindowCaption(const QString &)",
            $doc->widget()->topLevelWidget(), SLOT "setCaption(const QString &)" );

$toplevel->show();
$a->exec;


