#include <ktrader.h>
#include <kmainwindow.h>
#include <kfileitem.h>
#include <kcmdlineargs.h>
#include <qptrlist.h>

#include "handlers.cpp"

static void marshall_QValueListQCString(Marshall *m) {
    switch(m->action()) {
      case Marshall::FromSV:
	{
	    SV *sv = m->var();
	    if(!SvROK(sv) || SvTYPE(SvRV(sv)) != SVt_PVAV ||
		av_len((AV*)SvRV(sv)) < 0) {
		m->item().s_voidp = 0;
		break;
	    }
	    AV *list = (AV*)SvRV(sv);
	    int count = av_len(list);
	    QValueList<QCString> *valuelist = new QValueList<QCString>;
	    int i;
	    for(i = 0; i <= count; i++) {
		SV **item = av_fetch(list, i, 0);
		if(!item || !SvOK(*item)) {
		    valuelist->append(QCString());
		    continue;
		}

		valuelist->append(QCString(SvPV_nolen(*item)));
	    }

	    m->item().s_voidp = valuelist;
	    m->next();

	    if(m->cleanup()) {
		av_clear(list);
                COP *cop = in_constructor ? cxstack[cxstack_ix-3].blk_oldcop : cxstack[cxstack_ix].blk_oldcop;
		for(QValueListIterator<QCString> it = valuelist->begin();
		    it != valuelist->end();
		    ++it) {
		    uint len =  (*it).length();
		    SV* svs = newSVpvn((const char*)*it, len);
		    if(!(cop->op_private & HINT_BYTES) && not_ascii((const char*)*it,len))
		    {
			#if PERL_VERSION == 6 && PERL_SUBVERSION == 0
			QTextCodec* c = QTextCodec::codecForMib(106); // utf8
			if(c->heuristicContentMatch((const char*)*it,len) >= 0)
			#else
			if(is_utf8_string((U8 *)(const char*)*it,len))
			#endif
			    SvUTF8_on(svs);
		    }
		    av_push(list, svs);
                }
		delete valuelist;
	    }
	}
	break;
      case Marshall::ToSV:
	{
	    QValueList<QCString> *valuelist = (QValueList<QCString>*)m->item().s_voidp;
	    if(!valuelist) {
		sv_setsv_mg(m->var(), &PL_sv_undef);
		break;
	    }
	    COP *cop = cxstack[cxstack_ix].blk_oldcop;
	    AV *av = newAV();
	    {
		SV *rv = newRV_noinc((SV*)av);
		sv_setsv_mg(m->var(), rv);
		SvREFCNT_dec(rv);
	    }

	    for(QValueListIterator<QCString> it = valuelist->begin();
		it != valuelist->end();
		++it) {
		uint len = (*it).length();
		SV* svs = newSVpvn((const char *)*it, len);
		if(!(cop->op_private & HINT_BYTES) && not_ascii((const char *)*it,len))
		{
		    #if PERL_VERSION == 6 && PERL_SUBVERSION == 0
		    QTextCodec* c = QTextCodec::codecForMib(106); // utf8
		    if(c->heuristicContentMatch((const char*)*it,len) >= 0)
		    #else
		    if(is_utf8_string((U8 *)(const char*)*it,len))
		    #endif
			SvUTF8_on(svs);
		}
		av_push(av, svs);
            }

	    if(m->cleanup())
		delete valuelist;
	}
	break;
      default:
	m->unsupported();
	break;
    }
}

void marshall_KCmdLineOptions(Marshall *m) {
    switch(m->action()) {
      case Marshall::FromSV:
	{
	    SV *sv = m->var();
	    if(!SvROK(sv) || SvTYPE(SvRV(sv)) != SVt_PVAV ||
		av_len((AV*)SvRV(sv)) < 0) {
		m->item().s_voidp = 0;
		break;
	    }
	    AV *list = (AV*)SvRV(sv);
	    int ix = av_len(list);

	    KCmdLineOptions *cmdLineOptions;
	    New(0, cmdLineOptions, ix+2, KCmdLineOptions);
	    int i;
	    for(i = 0; i <= ix; i++) {
		SV **item = av_fetch(list, i, 0);
		if(!item || !SvROK(*item) || SvTYPE(SvRV(*item)) != SVt_PVAV ||
		av_len((AV*)SvRV(*item)) != 2) {
		    warning("Skipping malformed KCmdLineOptions entry. Index: %d\n", i);
		    continue;
		}
		SV **name, **descr, **def;
                AV *opt =(AV*) SvRV(*item);
		name = av_fetch(opt, 0, 0);
		descr = av_fetch(opt, 1, 0);
		def = av_fetch(opt, 2, 0);

		cmdLineOptions[i].name = (name && SvPOK(*name)) ? SvPV_nolen(*name) : 0;
		cmdLineOptions[i].description = (descr && SvPOK(*descr)) ? SvPV_nolen(*descr) : 0;
		cmdLineOptions[i].def = (def && SvPOK(*def)) ? SvPV_nolen(*def) : 0;
	    }
	    cmdLineOptions[i].name = 0;
	    cmdLineOptions[i].description = 0;
	    cmdLineOptions[i].def = 0;
	    m->item().s_voidp = cmdLineOptions;
	    m->next();
//	    if(m->cleanup()) Safefree((void*)cmdLineOptions);
	}
	break;
      default:
	m->unsupported();
	break;
    }
}

void marshall_KTraderOfferList(Marshall *m) {
    switch(m->action()) {
      case Marshall::ToSV:
	{
	    bool hasMagic = false;
            KTrader::OfferList *valuelist = (KTrader::OfferList*)m->item().s_voidp;
	    if(!valuelist) {
		sv_setsv_mg(m->var(), &PL_sv_undef);
                break;
            }
	    AV *av = newAV();
	    {
		SV *rv = newRV_noinc((SV*)av);
		sv_setsv_mg(m->var(), rv);
		SvREFCNT_dec(rv);
	    }
	    for(KTrader::OfferListIterator it = valuelist->begin();
		it != valuelist->end();
		++it) {
                KService::Ptr* offer = new KService::Ptr(*it); // keep refcount high
                SV* rv = newSV(0);
                SV* sv = newSV(0);
                sv_setref_pv(rv, "KDE::_internal::KServicePtr", (void*)offer);
                sv_magic(sv, rv, PERL_MAGIC_tiedscalar, Nullch, 0);
		av_push(av, sv);
            }
	    if(m->cleanup())
		delete valuelist;
	}
	break;
      default:
	m->unsupported();
	break;
    }
}

MARSHALL_QPTRLIST( QPtrListKMainWindow, QPtrList<KMainWindow>, KMainWindow, " KDE::MainWindow", FALSE )
MARSHALL_QPTRLIST( QPtrListFileItem, QPtrList<KFileItem>, KFileItem, " KDE::FileItem", FALSE )

TypeHandler KDE_handlers[] = {
    { "QCStringList", marshall_QValueListQCString },
    { "QCStringList&", marshall_QValueListQCString },
    { "QCStringList*", marshall_QValueListQCString },
    { "QValueList<QCString>", marshall_QValueListQCString },
    { "QValueList<QCString>&", marshall_QValueListQCString },
    { "QValueList<QCString>*", marshall_QValueListQCString },
    { "KCmdLineOptions", marshall_KCmdLineOptions },
    { "KCmdLineOptions&", marshall_KCmdLineOptions },
    { "KCmdLineOptions*", marshall_KCmdLineOptions },
    { "KFileItemList", marshall_QPtrListFileItem },
    { "KFileItemList&", marshall_QPtrListFileItem },
    { "KFileItemList*", marshall_QPtrListFileItem },
    { "KMainWindowList", marshall_QPtrListKMainWindow },
    { "KMainWindowList&", marshall_QPtrListKMainWindow },
    { "KMainWindowList*", marshall_QPtrListKMainWindow },
    { "KTrader::OfferList", marshall_KTraderOfferList },
    { "KTrader::OfferList&", marshall_KTraderOfferList },
    { "KTrader::OfferList*", marshall_KTraderOfferList },
    { 0, 0 }
};
