#!/usr/bin/perl -w
use strict;
use Qt::debug;
package CheckLists;
use Qt;
use Qt::isa qw(Qt::Widget);
use Qt::slots
	copy1to2 => [],
	copy2to3 => [];
use Qt::attributes qw(
	label
	lv1
	lv2
);

sub NEW {
    shift->SUPER::NEW(@_);

		my $lay = Qt::HBoxLayout(this);
		$lay->setMargin(5);

		# create a widget which layouts its childs in a column
		my $vbox1 = Qt::VBoxLayout($lay);
		$vbox1->setMargin(5);

		# First child: a Label
		$vbox1->addWidget(Qt::Label('Check some items!', this));

		# Second child: the ListView
		lv1 = Qt::ListView(this);
		$vbox1->addWidget(lv1);
		lv1->addColumn('Items');
		lv1->setRootIsDecorated(1);

		# create a list with 4 ListViewItems which will be parent items of other ListViewItems
		my @parentList;

		push @parentList, Qt::ListViewItem(lv1, 'Parent Item 1');
		push @parentList, Qt::ListViewItem(lv1, 'Parent Item 2');
		push @parentList, Qt::ListViewItem(lv1, 'Parent Item 3');
		push @parentList, Qt::ListViewItem(lv1, 'Parent Item 4');

		my $item = 0;
		my $num = 1;
		# go through the list of parent items...
    	foreach $item (@parentList) {
			$item->setOpen(1);
			# ...and create 5 checkable child ListViewItems for each parent item
    		for (my $i = 0; $i < 5; $i++) {
				my $str = sprintf('%s. Child of Parent %s', $i, $num);
				Qt::CheckListItem($item, $str, &Qt::CheckListItem::CheckBox);
			}
			$num = $num + 1;
		}

		# Create another widget for layouting
		my $tmp = Qt::VBoxLayout($lay);
		$tmp->setMargin(5);

		# create a pushbutton
		my $copy1 = Qt::PushButton('  ->  ', this);
		$tmp->addWidget($copy1);
		$copy1->setMaximumWidth($copy1->sizeHint()->width);
		# connect the SIGNAL clicked() of the pushbutton with the SLOT copy1to2()
		Qt::Object::connect($copy1, SIGNAL('clicked()'), this, SLOT('copy1to2()'));

		# another widget for layouting
		my $vbox2 = Qt::VBoxLayout($lay);
		$vbox2->setMargin(5);

		# and another label
		$vbox2->addWidget(Qt::Label('Check one item!', this));

		# create the second listview
		lv2 = Qt::ListView(this);
		$vbox2->addWidget(lv2);
		lv2->addColumn('Items');
		lv2->setRootIsDecorated(1);

		# another widget needed for layouting only
		$tmp = Qt::VBoxLayout($lay);
		$tmp->setMargin(5);

		# create another pushbutton...
		my $copy2 = Qt::PushButton('  ->  ', this);
		$lay->addWidget( $copy2 );
		$copy2->setMaximumWidth($copy2->sizeHint()->width());
		# ...and connect its clicked() SIGNAL to the copy2to3() SLOT
		Qt::Object::connect($copy2, SIGNAL('clicked()'), this, SLOT('copy2to3()'));

		$tmp = Qt::VBoxLayout($lay);
		$tmp->setMargin(5);

		# and create a label which will be at the right of the window
		label = Qt::Label('No Item yet...', this);
		$tmp->addWidget(label);
	}

	# SLOT copy1to2()
	#
	# Copies all checked ListViewItems from the first ListView to
	# the second one, and inserts them as Radio-ListViewItem.
	sub copy1to2
	{
		lv2->clear();
		my $it = Qt::ListViewItemIterator(lv1);
		# Insert first a controller Item into the second ListView. Always if Radio-ListViewItems
		# are inserted into a Listview, the parent item of these MUST be a controller Item!
		my $item = Qt::CheckListItem(lv2, 'Controller', &Qt::CheckListItem::Controller );
		$item->setOpen(1);

		# iterate through the first ListView...
		while ($it->current()) {
			# ...check state of childs, and...
			if ( $it->current()->parent ) {
				#  ->..if the item is checked...
				if ($it->current()->isOn()) {
					# ...insert a Radio-ListViewItem with the same text into the second ListView
					Qt::CheckListItem($item, $it->current()->text(0), &Qt::CheckListItem::RadioButton)
				}
			}
			$it += 1;
		}

		if ($item->firstChild) {
			$item->firstChild->setOn(1);
		}
	}


	# SLOT copy2to3()
	#
	# Copies the checked item of the second ListView into the
	# Label at the right.
	sub copy2to3
	{
		# create an iterator which operates on the second ListView
		my $it = Qt::ListViewItemIterator(lv2);

		label->setText('No Item checked');

		# iterate through the second ListView...
		while ($it->current) {
			# ...check state of childs, and...
			if ( $it->current->parent) {
				# ...if the item is checked...
				if ($it->current->isOn) {
					# ...set the text of the item to the label
					label->setText($it->current->text(0));
				}
			}
			$it += 1;
		}
	}

package main;
use Qt;
use CheckLists;

my $a = Qt::Application(\@ARGV);
my $checklists = CheckLists;
$a->setMainWidget($checklists);
$checklists->setCaption("PerlQt Example - CheckLists");
$checklists->show;
exit $a->exec;

